@@ -1,7 +1,7 @@
 /*
    Kickshaw - A Menu Editor for Openbox
 
-   Copyright (c) 2010–2022        Marcus Schätzle
+   Copyright (c) 2010–2024        Marcus Schätzle
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -97,7 +97,7 @@ static void hide_or_deactivate_widgets (void)
 
 */
 
-void one_of_the_enter_values_buttons_pressed (const char *origin)
+void one_of_the_enter_values_buttons_pressed (const gchar *origin)
 {
     GSList *enter_values_user_settings_loop;
     // AD = adding and deleting, indicating that the enums are only used here
@@ -109,7 +109,7 @@ void one_of_the_enter_values_buttons_pressed (const char *origin)
     g_autofree gchar *new_menu_element = NULL;
 
     FOREACH_IN_LIST (ks.enter_values_user_settings, enter_values_user_settings_loop) {
-        g_autofree gchar **parameter = g_strsplit (enter_values_user_settings_loop->data, ":", -1);
+        g_auto(GStrv) parameter = g_strsplit (enter_values_user_settings_loop->data, ":", -1);
 
         if (STREQ (origin, "done") && STREQ (parameter[AD_FIELD], "new menu element")) {
             // "value" exists only temporary
@@ -117,11 +117,11 @@ void one_of_the_enter_values_buttons_pressed (const char *origin)
         }
         else if (STREQ (origin, "reset")) {
             if (STREQ (parameter[AD_FIELD], "new menu element")) {
-                const gchar *label_element = (STREQ (parameter[AD_VALUE], "menu")) ? "Menu" : 
-                                             ((STREQ (parameter[AD_VALUE], "pipe menu")) ? "Pipe Menu" : "Item");
-                g_autofree gchar *default_label_txt = g_strdup_printf ("New %s", label_element);
+                const gchar *label_txt = (STREQ (parameter[AD_VALUE], "menu")) ? _("New Menu") : 
+                                         ((STREQ (parameter[AD_VALUE], "pipe menu")) ? _("New Pipe Menu") : 
+                                          _("New Item"));
 
-                gtk_entry_set_text (GTK_ENTRY (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY]), default_label_txt);
+                gtk_entry_set_text (GTK_ENTRY (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY]), label_txt);
 
                 // To keep the code simple, the rest of this if-clause resets fields even if they are currently not visible.
                 gtk_style_context_remove_class (gtk_widget_get_style_context (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY]), 
@@ -226,7 +226,7 @@ void one_of_the_enter_values_buttons_pressed (const char *origin)
             const gchar *menu_id = gtk_entry_get_text (GTK_ENTRY (ks.entry_fields[MENU_ID_ENTRY]));
 
             if (G_UNLIKELY (g_slist_find_custom (ks.menu_ids, menu_id, (GCompareFunc) strcmp))) {
-                show_errmsg ("The chosen menu ID already exists. Please choose another one.");
+                show_errmsg (_("The chosen menu ID already exists. Please choose another one."));
                 missing_entry_or_error = TRUE;
             }
             else if (!missing_entry_or_error) {
@@ -257,10 +257,10 @@ void one_of_the_enter_values_buttons_pressed (const char *origin)
         GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (ks.treeview));
         gtk_tree_selection_select_iter (selection, &to_be_selected_iter);
 
-        push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+        push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
         activate_change_done ();
-        row_selected ();                     /* ... the toolbar buttons and 
-                                                    application menu's undo/redo items get the correct sensitivity settings. */
+        row_selected ();                              /* ... the toolbar buttons and 
+                                                         application menu's undo/redo items get the correct sensitivity settings. */
     }
 }
 
@@ -336,9 +336,9 @@ static void add_new_execution (const gchar       *new_menu_element,
         }
     }
 
-    // Set element visibility of menus, pipe menus, items and separators.
+    // Set element visibility of menus, pipe menus, items, and separators.
 
-    if (regex_match_simple ("menu|pipe menu|item.*|separator", new_menu_element, PCRE2_UTF, 0)) {
+    if (g_regex_match_simple ("menu|pipe menu|item.*|separator", new_menu_element, 0, 0)) {
         element_visibility_txt = "visible";
         /*
             path == NULL means that the new menu element has been added at toplevel;
@@ -439,8 +439,13 @@ static void highlight_menu_ID_entry_field_for_double_menu_IDs (G_GNUC_UNUSED gpo
 
 void add_new (const gchar *new_menu_element_plus_opt_suppl)
 {
+    // This makes sure that static analyzers don't assume that new_menu_element_plus_opt_suppl can be NULL.
+#ifndef G_DISABLE_ASSERT
+    g_assert (new_menu_element_plus_opt_suppl);
+#endif
+
     gboolean same_level = TRUE; // Default
-    const gchar *new_menu_element = (!regex_match_simple ("item.*", new_menu_element_plus_opt_suppl, PCRE2_UTF, 0)) ? 
+    const gchar *new_menu_element = (!g_regex_match_simple ("item.*", new_menu_element_plus_opt_suppl, 0, 0)) ? 
                                     new_menu_element_plus_opt_suppl : "item";
 
     if (!streq_any (new_menu_element, "menu", "pipe menu", "item", "separator", NULL)) {
@@ -453,10 +458,10 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
         // NULL = iter of added menu element is not needed for later use.
         add_new_execution (new_menu_element, NULL, same_level);
 
-        push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+        push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
         activate_change_done ();
-        row_selected ();                     /* ... the toolbar buttons and 
-                                                    application menu's undo/redo items get the correct sensitivity settings. */
+        row_selected ();                              /* ... the toolbar buttons and 
+                                                         application menu's undo/redo items get the correct sensitivity settings. */
     }
     else {
         gchar *field_value_pair;
@@ -467,18 +472,27 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
         */
         if (STREQ (ks.txt_fields[TYPE_TXT], "menu")) {
             GtkWidget *dialog;
-            g_autofree gchar *label_txt;
+            const char *label_txt;
 
             gint result;
 
             enum { CANCEL = 1, SAME_LEVEL, INSIDE_MENU };
 
-            label_txt = g_strdup_printf ("Insert new %s on the same level or into the currently selected menu?", 
-                                         new_menu_element);
+            if (STREQ (new_menu_element, "menu")) {
+                label_txt = _("Insert new menu on the same level or into the currently selected menu?");
+            }
+            else if (STREQ (new_menu_element, "pipe menu")) {
+                label_txt = _("Insert new pipe menu on the same level or into the currently selected menu?");
+            }
+            else if (STREQ (new_menu_element, "item")) {
+                label_txt = _("Insert new item on the same level or into the currently selected menu?");
+            }
+            else { // separator
+                label_txt = _("Insert new separator on the same level or into the currently selected menu?");
+            }
 
-			// FALSE = don't show header barshow header bar, TRUE = dialog is shown immediately
-            create_dialog (&dialog, "Same Level or Inside Menu?", "dialog-question", label_txt, 
-                           "_Cancel", "_Same Level", "_Inside Menu", FALSE, TRUE); // TRUE = dialog is shown immediately.
+            create_dialog (&dialog, _("Same Level or Inside Menu?"), "dialog-question", label_txt, 
+                           C_("Cancel|No File Dialogue", "_Cancel"), _("_Same Level"), _("_Inside Menu"), SHOW_IMMEDIATELY);
 
             result = gtk_dialog_run (GTK_DIALOG (dialog));
             gtk_widget_destroy (dialog);
@@ -503,9 +517,9 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
         }
 
         if (!STREQ (new_menu_element, "separator")) {
-            g_autofree gchar *enter_values_markup, *new_menu_element_headline, *default_label_txt;
-            const gchar *label_element = (STREQ (new_menu_element, "menu")) ? "Menu" : 
-                                         ((STREQ (new_menu_element, "pipe menu")) ? "Pipe Menu" : "Item");
+            const gchar *headline;
+            g_autofree gchar *enter_values_markup;
+            const gchar *default_label_txt;
             const gboolean menu_or_pipe_menu = (!STREQ (new_menu_element, "item"));
 
 
@@ -537,7 +551,7 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
                                                        ACTION_OPTION_COMBO_ITEM, ks.actions[action_cnt], -1);
                 }
 
-                if (regex_match_simple ("item\\+.*", new_menu_element_plus_opt_suppl, PCRE2_UTF, 0)) {
+                if (g_regex_match_simple ("item\\+.*", new_menu_element_plus_opt_suppl, 0, 0)) {
                     g_autofree gchar *action = extract_substring_via_regex (new_menu_element_plus_opt_suppl, "(?<=\\+).*");
 
                     gtk_combo_box_set_active_id (GTK_COMBO_BOX (ks.new_action_option_widgets[NEW_ACTION_OPTION_COMBO_BOX]), action);
@@ -560,13 +574,17 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
 
             hide_or_deactivate_widgets ();
 
-            new_menu_element_headline = g_strdup (new_menu_element);
-            new_menu_element_headline[0] = g_ascii_toupper (new_menu_element_headline[0]);
-            if (STREQ (new_menu_element, "pipe menu")) {
-                new_menu_element_headline[5] = g_ascii_toupper (new_menu_element_headline[5]);
+            if (STREQ (new_menu_element, "menu")) {
+                headline = _("Enter Values for New Menu");
             }
-            enter_values_markup = g_strdup_printf ("<span font_desc='%i'>Enter Values for New %s</span>", 
-                                                    ks.font_size + 2, new_menu_element_headline);
+            else if (STREQ (new_menu_element, "pipe menu")) {
+                headline = _("Enter Values for New Pipe Menu");
+            }
+            else { // item
+                headline = _("Enter Values for New Item");
+            }
+            enter_values_markup = g_strdup_printf ("<span font_desc='%i'>%s</span>", 
+                                                   ks.font_size + 2, headline);
             gtk_label_set_markup (GTK_LABEL (ks.enter_values_label), enter_values_markup);
 
             gtk_widget_show (ks.enter_values_box);
@@ -596,7 +614,9 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
 
             field_value_pair = g_strdup_printf ("new menu element:%s", new_menu_element);
             ks.enter_values_user_settings = g_slist_prepend (ks.enter_values_user_settings, field_value_pair);
-            default_label_txt = g_strconcat ("New ", label_element, NULL);
+            default_label_txt = (STREQ (new_menu_element, "menu")) ? _("New Menu") : 
+                                ((STREQ (new_menu_element, "pipe menu")) ? _("New Pipe Menu") : 
+                                 _("New Item"));
             gtk_entry_set_text (GTK_ENTRY (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY]), default_label_txt);
             // In case that currently a (pipe) menu or item without label is selected.
             gtk_widget_set_sensitive (ks.entry_fields[MENU_ELEMENT_OR_VALUE_ENTRY], TRUE);
@@ -611,7 +631,7 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
 
                 // Menu IDs have to be unique, so the list of menu IDs has to be checked for existing values.
                 while (TRUE) {
-                    g_autofree gchar *menu_ID_loop = g_strdup_printf ("new menu %i", menu_id_index++);
+                    g_autofree gchar *menu_ID_loop = g_strdup_printf (_("new menu %i"), menu_id_index++);
 
                     if (!g_slist_find_custom (ks.menu_ids, menu_ID_loop, (GCompareFunc) strcmp)) {
                         menu_ID = g_strdup (menu_ID_loop);
@@ -627,7 +647,7 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
                 gtk_entry_set_text (GTK_ENTRY (ks.entry_fields[EXECUTE_ENTRY]), "");
 
                 if (G_UNLIKELY (g_slist_find_custom (ks.menu_ids, "", (GCompareFunc) strcmp))) {
-                    g_string_printf (ks.menu_ID_field_txt, " Menu ID (<span foreground='%s'>*</span>): ", ks.blue_hue);
+                    g_string_printf (ks.menu_ID_field_txt, _(" Menu ID (<span foreground='%s'>*</span>): "), ks.blue_hue);
 
                     gtk_label_set_markup (GTK_LABEL (ks.entry_labels[MENU_ID_ENTRY]), ks.menu_ID_field_txt->str);
                     gtk_label_set_markup (GTK_LABEL (ks.mandatory), ks.mandatory_empty_string_txt->str);
@@ -646,10 +666,10 @@ void add_new (const gchar *new_menu_element_plus_opt_suppl)
             // NULL = iter of added menu element is not needed for later use.
             add_new_execution ("separator", NULL, same_level);
 
-            push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+            push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
             activate_change_done ();
-            row_selected ();                     /* ... the toolbar buttons and 
-                                                    application menu's undo/redo items get the correct sensitivity settings. */
+            row_selected ();                              /* ... the toolbar buttons and 
+                                                             application menu's undo/redo items get the correct sensitivity settings. */
         }
     }
 }
@@ -705,14 +725,14 @@ static void option_list_with_headlines (G_GNUC_UNUSED GtkCellLayout   *cell_layo
                         ACTION_OPTION_COMBO_ITEM, &action_option_combo_item, 
                         -1);
 
-    const gboolean headline = regex_match_simple ("Add|Choose", action_option_combo_item, PCRE2_UTF, PCRE2_ANCHORED);
+    const gboolean headline = g_regex_match_simple (_("Choose|Add"), action_option_combo_item, G_REGEX_CASELESS, 0);
 
     g_object_set (action_option_combo_box_renderer, 
                   "foreground-rgba", (headline) ? &((GdkRGBA) { 1.0, 1.0, 1.0, 1.0 }) : NULL, 
-                  "background-rgba", (g_str_has_prefix (action_option_combo_item, "Choose")) ? 
-                                     &((GdkRGBA) { 0.31, 0.31, 0.79, 1.0 }) : 
-                                     ((g_str_has_prefix (action_option_combo_item, "Add")) ? 
-                                     &((GdkRGBA) { 0.0, 0.0, 0.0, 1.0 }) : NULL),
+                  "background-rgba", (g_regex_match_simple (_("Add"), action_option_combo_item, G_REGEX_CASELESS, 0)) ? 
+                                     &((GdkRGBA) { 0.0, 0.0, 0.0, 1.0 }) : 
+                                     ((g_regex_match_simple (_("Choose"), action_option_combo_item, G_REGEX_CASELESS, 0)) ?  
+                                     &((GdkRGBA) { 0.31, 0.31, 0.79, 1.0 }) : NULL),
                   "sensitive", !headline, 
                   NULL);
 }
@@ -728,8 +748,7 @@ static guint8 generate_action_option_combo_box_items_for_Exe_and_snotify_opts (c
     const gboolean execute = STREQ (options_type, "Execute");
     GtkTreeIter parent, action_option_combo_box_new_iter;
     const guint8 number_of_opts = (execute) ? NUMBER_OF_EXECUTE_OPTS : NUMBER_OF_STARTUPNOTIFY_OPTS;
-    const guint allocated_size = sizeof (gboolean) * number_of_opts;
-    g_autofree gboolean *opts_exist = g_malloc0 (allocated_size); // Initialize all elements to FALSE.
+    g_autofree gboolean *opts_exist = g_malloc0 (sizeof (gboolean) * number_of_opts); // Initialize all elements to FALSE.
 
     guint8 number_of_added_opts = 0; // Start value
 
@@ -765,9 +784,8 @@ void generate_items_for_action_option_combo_box (const gchar *preset_choice)
 {
     GtkTreeIter action_option_combo_box_iter;
 
-    const gchar *bt_add_action_option_label_txt;
-
-    g_autofree gchar *add_inside_txt = NULL; // Default
+    const gint number_of_children_of_iter = gtk_tree_model_iter_n_children (ks.ts_model, &ks.iter);
+    const gchar *add_inside_txt = NULL; // Default
 
     guint8 number_of_added_actions = 0, number_of_added_action_opts = 0, number_of_added_snotify_opts = 0; // Defaults
 
@@ -777,13 +795,16 @@ void generate_items_for_action_option_combo_box (const gchar *preset_choice)
 
     create_combo_box ();
 
-    // Generate "Choose..." headline according to the text of bt_add_action_option_label.
-    bt_add_action_option_label_txt = gtk_label_get_text (GTK_LABEL (ks.bt_add_action_option_label));
+    // Generate "Choose..." headline according to the type of the selected menu element.
     gtk_list_store_insert_with_values (ks.action_option_combo_box_liststore, &action_option_combo_box_iter, -1, 
-                                       ACTION_OPTION_COMBO_ITEM,
-                                       (STREQ (bt_add_action_option_label_txt, "Action")) ? "Choose an Action" : 
-                                       (STREQ (bt_add_action_option_label_txt, "Option")) ? 
-                                       "Choose an Option" : "Choose an Action/Option", 
+                                       ACTION_OPTION_COMBO_ITEM, 
+                                       (STREQ (ks.txt_fields[TYPE_TXT], "item") || 
+                                       ((STREQ (ks.txt_fields[TYPE_TXT], "action")) && 
+                                        ((STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Reconfigure")) || 
+                                        (STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Execute") && number_of_children_of_iter == NUMBER_OF_EXECUTE_OPTS) || 
+                                        number_of_children_of_iter == 1))) // Exit, Restart, or SessionLogout
+                                       ? _("Choose an Action") : 
+                                       ((streq_any (ks.txt_fields[TYPE_TXT], "option", "option block", NULL)) ? _("Choose an Option") : _("Choose an Action/Option")), 
                                        -1);
 
     // Startupnotify options: enabled, name, wmclass, icon
@@ -792,7 +813,7 @@ void generate_items_for_action_option_combo_box (const gchar *preset_choice)
         number_of_added_snotify_opts = generate_action_option_combo_box_items_for_Exe_and_snotify_opts ("startupnotify");
     }
 
-    // Execute options: prompt, command and startupnotify
+    // Execute options: prompt, command, and startupnotify
     if ((STREQ (ks.txt_fields[TYPE_TXT], "action") && STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Execute")) ||
         (streq_any (ks.txt_fields[TYPE_TXT], "option", "option block", NULL) && 
         streq_any (ks.txt_fields[MENU_ELEMENT_TXT], "prompt", "command", "startupnotify", NULL))) {
@@ -824,10 +845,21 @@ void generate_items_for_action_option_combo_box (const gchar *preset_choice)
         the second one next to the current selection. If so, add headlines to the list to separate them from each other.
     */
     if (number_of_added_snotify_opts && number_of_added_action_opts) {
-        add_inside_txt = g_strdup ("Add Inside Startupnotify");
+        add_inside_txt = _("Add Inside Startupnotify");
     }
     else if (number_of_added_action_opts && number_of_added_actions) {
-        add_inside_txt = g_strdup_printf ("Add Inside Currently Selected %s Action", ks.txt_fields[MENU_ELEMENT_TXT]);
+        if (STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Execute")) {
+            add_inside_txt = _("Add Inside Currently Selected Execute Action");
+        }
+        else if (STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Exit")) {
+            add_inside_txt = _("Add Inside Currently Selected Exit Action");
+        }
+        else if (STREQ (ks.txt_fields[MENU_ELEMENT_TXT], "Restart")) {
+            add_inside_txt = _("Add Inside Currently Selected Restart Action");
+        }
+        else { // SessionLogout
+            add_inside_txt = _("Add Inside Currently Selected SessionLogut Action");
+        }
     }
 
     if (add_inside_txt) {
@@ -847,7 +879,7 @@ void generate_items_for_action_option_combo_box (const gchar *preset_choice)
                                            ((number_of_added_actions) ? 
                                            number_of_added_action_opts : number_of_added_snotify_opts) + 2, 
                                            ACTION_OPTION_COMBO_ITEM, (number_of_added_actions) ? 
-                                           "Add Additional Action" : "Add Next to Startupnotify", 
+                                           _("Add Additional Action") : _("Add Next to Startupnotify"), 
                                            -1);
     }
 
@@ -961,7 +993,7 @@ static void show_action_options (void)
         gtk_widget_grab_focus (ks.options_fields[COMMAND]);
     }
 
-    // "Exit" or "SessionLogout" action or "prompt" option (the latter for "Execute", "Exit" and "SessionLogout" actions)
+    // "Exit" or "SessionLogout" action or "prompt" option (the latter for "Execute", "Exit", and "SessionLogout" actions)
     if (streq_any (combo_choice, "Exit", "Prompt", "SessionLogout", NULL)) {
          if (!STREQ (combo_choice, "Prompt") || 
             (STREQ (combo_choice, "Prompt") && streq_any (ks.txt_fields[MENU_ELEMENT_TXT], "Exit", "SessionLogout", NULL))) {
@@ -1040,7 +1072,7 @@ void single_field_entry (void)
     /*
         Applies to 
         - the _single_ "Execute" options "command" and "prompt"
-        - the _single_ "startupnotify" options "name", "wmclass" and "icon"
+        - the _single_ "startupnotify" options "name", "wmclass", and "icon"
         - the "Restart" action and its option "command".
 
         If action fields are shown in conjunction with item fields, single field entry is not executed.
@@ -1369,10 +1401,10 @@ void action_option_insert (const gchar *origin)
        The item is selected after this function has returned to one_of_the_enter_values_buttons_pressed (), 
        and only after that row_selected () is called from there. */
     if (!gtk_widget_get_visible (ks.enter_values_box)) {
-        push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+        push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
         activate_change_done ();
-        row_selected ();                     /* ... the toolbar buttons and 
-                                                    application menu's undo/redo items get the correct sensitivity settings. */
+        row_selected ();                              /* ... the toolbar buttons and 
+                                                         application menu's undo/redo items get the correct sensitivity settings. */
     }
 }
 
@@ -1570,10 +1602,10 @@ void remove_rows (const gchar *origin)
     g_signal_handler_unblock (selection, ks.handler_id_row_selected);
 
     if (!streq_any (origin, "dnd", "load menu", NULL)) { // There might be further changes during Drag & Drop / loading a menu.
-        push_new_item_on_undo_stack (FALSE); // First update undo stack so ...
+        push_new_item_on_undo_stack (NOT_AFTER_SAVE); // First update undo stack so ...
         activate_change_done ();
-        row_selected ();                     /* ... the toolbar buttons and 
-                                                    application menu's undo/redo items get the correct sensitivity settings. */
+        row_selected ();                              /* ... the toolbar buttons and 
+                                                         application menu's undo/redo items get the correct sensitivity settings. */
     }
 
 #if !(GLIB_CHECK_VERSION(2,56,0))
